'use strict';

var clc = require('cli-color');
var config = require('../config');
var util = require('./util');
var log = require('./log');

var isBatch;
var isDebug;
var isTestMySelf;

var tester;
var testerName;

var cycles;
var times;
var proof;

var rates = [];
var seconds = [];
var callback;

var runner = {
	do: function (tester_, callback_) {
		var args = Array.prototype.slice.call(arguments);

		if (typeof tester_ === 'function') {
			runner.init(args);
			runner.go();
		}
		else {
			runner.config(args);
			runner.do.isDebug = isDebug;
			return runner.do;
		}
	},

	config: function(args){
		isBatch = util.getIsBatch(); // batch mode

		var ret;
		var isSetDebug = util.isSetDebug(args[0]);
		var isSetConfig = !isSetDebug;

		var config_ = isSetConfig ? args[0] : args[1];
		var debug_ = isSetDebug ? args[0] : args[1];

		if (!isBatch && debug_) { // set debug
			isDebug = typeof debug_ === 'object' ? debug_.isDebug : debug_;
			ret = true;
		}

		if (config_) { // customize config
			util.copy(config, config_);
			ret = true;
		}

		return ret;
	},

	init: function(args){
		tester = args[0];

		if (typeof args[1] === 'function') {
			callback = args[1];
		}

		isBatch = util.getIsBatch(); // batch mode
		typeof global.isTestMySelf !== 'undefined' && (isTestMySelf = global.isTestMySelf);

		isBatch && (isDebug = false);
		!isBatch && isTestMySelf && (isDebug = true);

		testerName = isBatch ? process.argv[2] : util.getTesterName(process.argv[1]);
		cycles = isDebug ? config.debug.cycles : config.cycles;
		times = isDebug ? config.debug.times : config.times;
		proof = util.addUp(times);

		log.init(isTestMySelf);
	},

	go: function(){
		setTimeout(function () {
			runner.runTest();
		}, config.waitingTime);
	},

	runTest: function () {
		var startTime = process.hrtime();
		var count = 0;
		var calc = 0;

		while (count < times) {
			count ++;

			(function(count){
				tester(testDone, count);
			})(count);
		}

		function testDone(count){
			calc += count;

			isDebug && log.do(clc.white, '---- [times %s/%s]', count, times);

			if (calc === proof) {
				var duration = process.hrtime(startTime);
				var seconds = duration[1] / 1e9 + duration[0];
				var rate = (config.loadTest ? config.loadTest.maxRequests : times) / seconds;

				setImmediate(function () {
					runner.done(null, seconds, rate);
				});
			}
		}
	},

	done: function(err, second, rate){
		var averageRate;
		var milliseconds;
		var totalTimes;
		var result;

		seconds.push(second);
		rates.push(rate);

		if (rates.length === cycles) {

			averageRate = parseInt(rates.slice().reduce(util.sum, 0) / rates.length);
			milliseconds = parseInt(seconds.slice().reduce(util.sum, 0) * 1000);
			totalTimes = config.loadTest ? config.loadTest.maxRequests + ' requests' : cycles * times + ' times';

			if (config.mode === 0) {
				result = averageRate + '\ttimes/sec' + ' (' + totalTimes + ', ' + milliseconds + ' ms)';
			}
			else {
				result = milliseconds + ' ms\t' + '(' + totalTimes + ', ' + averageRate + ' times/sec)';
			}

			if (isDebug) {
				log.do(clc.white, '==== [cycles %s/%s]', rates.length, cycles);
			}
			else {
				isBatch && process.stdout.write(clc.move.up(1));
				log.do(clc.green, util.tab(testerName), clc.cyan, result);
			}

			callback && callback();
		}
		else {
			!isBatch && isDebug && log.do(clc.white, '==== [cycles %s/%s]', rates.length, cycles);
			runner.runTest();
		}
	}
};

runner.do.isDebug = false;
runner.do.getLogs = log.getLogs;

module.exports = runner.do;
